#if ! defined( _CPFALLOCATOR_H_ )
#define _CPFALLOCATOR_H_

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include "YourTimeDateClass.h"
#include "PredictionGenerator.h"
#include "YourMatrix.h"
#include "Cpf.h"

#include "EOP.h"
#include "RotAngles.h"
#include "Offsets.h"

//-----------------------------------------------------------------------------
//  Class Definition
//-----------------------------------------------------------------------------
// This class provides the association between Consolidated Prediction Format
// (CPF) table entires and nodes of a predicted track. Generally these will
// form a 1 to 1 mapping, but this class may be modified to support other
// arrangements.
//
// Example: The following represents a code fragment to outline the process 
//  
// do 
// {
//    cpf = new cpf element
//    CpfAllocator orbitGen( .. cpf .. )
//    orbitGen.calculatePosition( startEpoch )
//    do for required period;
//    {
//       orbitGen.calculateNextPosition( epoch )
//       access and use ephemeris data referred to epoch
//    }
// }
//-----------------------------------------------------------------------------
class CpfAllocator : public PredictionGenerator
{

public:

    // Constructor
    // Arguments: stepSize  -  Node interval. For this version, this interval should
    //                         exactly match the data interval used in the provided CPF file.
    //            element   -  A prediction element i.e. an entry from CPF table 
    CpfAllocator( YourTimeIntervalClass stepSize, const Cpf* element, const EOP* eop );

    // Destructor
    virtual ~CpfAllocator();

    // Method to update the "element" data.
    void CpfAllocator::setElement( const Cpf* cpf );

    // Method to calculate the position and velocity at the next orbital node. 
    // Arguments: epoch - calculated epoch of the next node, 1 time interval after the last node.  
    virtual void calculateNextPosition( YourTimeDateClass &epoch );

    // A method to generate position and velocity predictions at a given time.
    // Arguments: epoch  -  nominal epoch for calculating the orbital position and velocity.
    // Note: Since the node is mapped to the available data, the actual epoch used after
    // this call will be at the closest node time prior to epoch, or the first available.
    // A call to calculateNextPosition provides the actual time.
    virtual void calculatePosition( YourTimeDateClass epoch );

    //===============
    // Access methods
    //===============

    // Access method for position, velocity and aberration correction data.
    YourMatrix<double>* posVelCor() const { return m_pvcMatrix;};

    // Access method for target Offset anmd rotation angle data (if available).
    YourMatrix<double>* targetOffsets() const { return m_offsetMatrix;};

    // Access method for accuracy/runoff data (if available).
    std::vector<double>* accuracy() const { return m_runoffSet;};

    // Access method for transponder data (if available).
    std::vector<double>* transponder() const { return m_transponderSet;};

    // Access method for bidriectional flag.
    bool biDirectional() const {return m_biDirectional;};

private:

    YourMatrix<double>* m_offsetMatrix;
    YourMatrix<double>* m_pvcMatrix;
    std::vector<double>* m_transponderSet;
    std::vector<double>* m_runoffSet;

    TargetOffsets*    m_offsets;
    RotationAngles*   m_rotAngles;
    Cpf::Transponder* m_transponder;
    Cpf::Accuracy*    m_runoffs;
    YourTimeDateClass     m_cpfEpoch;

    bool m_biDirectional;

};  // class CpfAllocator

#endif // _CPFALLOCATOR_H_
