#include "EOP.h"
#include "yourException.h"
#include <math.h>

/******************************************************************************
 Constructor 1
******************************************************************************/
EOP::EOP()
{
    m_dataOk = false;
    m_fileOk = true;
}
 
/******************************************************************************
 Constructor 2
******************************************************************************/
EOP::EOP( const std::string& filename )
{
    m_filename = filename;
    m_dataOk = false;
    m_fileOk = readFile();
}

/******************************************************************************
 Destructor
******************************************************************************/
EOP::~EOP()
{
}

/******************************************************************************
 This method appends a set of earth rotation data, ie polar motion
 x position (arcseconds), y position(arcseconds) and ut1-utc  at a given epoch.
******************************************************************************/
void EOP::add(YourTimeDateClass epoch, double xPolar, double yPolar, YourTimeIntervalClass ut1_utc)
{
    Eop eop;
    eop.epoch = epoch;
    eop.xPolar = RAD_PER_ARCSEC * xPolar;
    eop.yPolar = RAD_PER_ARCSEC * yPolar;
    m_aEop.push_back( eop );
    m_dataOk = true;
    return;
}


/******************************************************************************
 Method to read the ERP/EOP file obtained from USNO as a SER7 file or as
 IERS Bulletin A file.
******************************************************************************/
bool EOP::readFile() 
{
    if( m_filename.empty() )
        return false;

    // Try to open the file. 
    YourFile f;
    if( !f.fopen(m_filename.c_str(), std::ios::in) )
    {
        return false;
    }

    // File exists so read an EOP record, one line at a time.
    // Allow a line length of up to 100 characters.
    try
    {
        f.setLines(1, 100);
        std::string line;
        Eop eop;
        do
        {
            f >> line;
            if( !f.eof() )
            {
                if( parseRecord( line, eop ) )
                {
                    m_aEop.push_back( eop );
                    m_dataOk = true;
                }
            }
        
        }while( !f.eof() );
        return true;
    }
    catch( ... )
    {
        return false;
    }
}

/******************************************************************************
// The IERS EOP file from USNO (Ser7) has the following format;
//    Prediction flag - 'P' if predicted, ' ' if measured
//    Year -  yy
//    Month - mm
//    Day -   dd
//    Modified Julian Date - nnnnn
//    X-Pole        - float, arcsecs
//    X-Pole error  - float
//    Y-Pole        - float
//    Y-Pole error  - float
//    UT1-UTC       - float, seconds
//    UT1-UTC error - float
// The IERS Bulletin A EOP file from has the following format;
//    Year -  yy
//    Month - mm
//    Day -   dd
//    Modified Julian Date - nnnnn
//    X-Pole        - float, arcsecs
//    Y-Pole        - float, arcsecs
//    UT1-UTC       - float, seconds
// There may be leading records that do not meet this format.
// Records are generally in time order and are normally at 1-day intervals.
******************************************************************************/
bool EOP::parseRecord( const std::string& line, Eop& eop )
{
    if( line.empty() || line.size() < 3 )
        return false;

    std::istringstream s;
    s.str( line );

    std::string tmp;
    std::vector<std::string> tokens;
    while( s >> tmp )
    {
        tokens.push_back(tmp);
    }

    if( tokens[0] == "P" || tokens[0] == "p" )
        tokens.erase( tokens.begin() );

    if( tokens.size() == 10 )
    {
        tokens.erase(tokens.begin()+9);
        tokens.erase(tokens.begin()+7);
        tokens.erase(tokens.begin()+5);
    }
    if( tokens.size() != 7 )
        return false;

    std::string ss;
    for( long i = 0; i < 7; i++ )
        ss += tokens[i] + " ";
    s.str(ss);
    bool ok = true;
    long  year  = convert<long>(   s, ok ); if( !ok ) return false;
    long  month = convert<long>(   s, ok ); if( !ok ) return false;
    long  day   = convert<long>(   s, ok ); if( !ok ) return false;
    long  mjd   = convert<long>(   s, ok ); if( !ok ) return false;
    double xp   = convert<double>( s, ok ); if( !ok ) return false;
    double yp   = convert<double>( s, ok ); if( !ok ) return false;
    double dut  = convert<double>( s, ok ); if( !ok ) return false;

    if( year < 50  ) year += 2000;
    else if( year < 100 ) year += 1900;

    YourTimeDateClass date;
    date.setDate(year, month, day, 0, 0, 0, 0, 0 );
    eop.epoch = YourTimeDateClass::modJulianDate( mjd );
    if( eop.epoch != date )
        yourTHROW( "Error parsing EOP record" );

    eop.xPolar  = RAD_PER_ARCSEC * xp;
    eop.yPolar  = RAD_PER_ARCSEC * yp;
    eop.ut1_utc = YourTimeIntervalClass::seconds(dut);
    return true;
}


/******************************************************************************
 This subroutine uses linear interpolation to compute polar motion
 x position (arcseconds), y position(arcseconds) and ut1-utc at a given epoch.
******************************************************************************/
Eop EOP::get( const YourTimeDateClass& epoch )const
{
    Eop eop;
    double xp,yp;
    YourTimeIntervalClass dut;
    get( epoch, xp, yp, dut );
    eop.epoch = epoch;
    eop.xPolar = xp;
    eop.yPolar = yp;
    eop.ut1_utc = dut;
    return eop;
}

/******************************************************************************
 This subroutine uses linear interpolation or extrapolation
 of earth rotation data held in the given file to compute polar motion
 x position (arcseconds), y position(arcseconds) and 
 ut1-utc  at a given epoch.

 The function returns FALSE if the caller supplies an epoch outside the 
 range of available EOP data. The function does not extrapolate data.
******************************************************************************/
bool EOP::get( const YourTimeDateClass& epoch,
               double& xPolar, double& yPolar, YourTimeIntervalClass& ut1_utc)const
{
    xPolar  = yPolar = 0.0;
    ut1_utc = YourTimeIntervalClass::seconds(0);

    // Some data are available.
    if( !m_aEop.empty() )
    {
        std::vector<Eop>::const_iterator i;
        for( i = m_aEop.begin(); i < m_aEop.end() - 1; i++ )
        {
            double d1 = YourTimeIntervalClass::days(        epoch - i->epoch );
            double d2 = YourTimeIntervalClass::days( (i+1)->epoch -    epoch );
            double d3 = YourTimeIntervalClass::days( (i+1)->epoch - i->epoch );
            if( fabs(d1) <= fabs(d3) && fabs(d2) <= fabs(d3) )
            {
                double f = 0.0;
                if( fabs(d3) > 0.0 ) 
                    f = d1 / d3;
                xPolar  =  ( (i+1)->xPolar  - i->xPolar  ) * f + i->xPolar;
                yPolar  =  ( (i+1)->yPolar  - i->yPolar  ) * f + i->yPolar;
                ut1_utc =  ( (i+1)->ut1_utc - i->ut1_utc ) * f + i->ut1_utc;
                return true;
            }
        }
    }

    return false;
}

/******************************************************************************
 This subroutine uses linear interpolation or extrapolation
 of earth rotation data held in the given file to compute  
 ut1-utc (as a precision interval) at a given epoch. 
 A default zero length interval is returned if no data is available. 
******************************************************************************/
YourTimeIntervalClass EOP::ut1_utc(const YourTimeDateClass& epoch ) const
{
    double xp, yp;
    YourTimeIntervalClass ut1utc;
    if( get( epoch, xp, yp, ut1utc) )
        return ut1utc;
    else
        return YourTimeIntervalClass::seconds(0);
}

/******************************************************************************
Record type 70 Earth orientation (For space-fixed mode, 
as needed, typically once a day)

 1-2 A2 Record Type (= "70")
I5 Modified Julian Date (MJD)
I6 Seconds of Day (UTC)
F8.5 X pole (arcseconds)
F8.5 Y pole (arcseconds)
F10.6 dUTC (seconds)
******************************************************************************/
void EOP::addCpfRecord( const std::string& line )
{
    Eop eop;
    std::istringstream s;
    s.str( line );

    bool ok = true;
    std::string recType = convert<std::string>( s, ok );
    if( recType != "70" )
        yourTHROW( "Error reading CPF record 70.");

    double mjd  = convert<double>( s, ok );
    double secs = convert<double>( s, ok );
    double xp   = convert<double>( s, ok );
    double yp   = convert<double>( s, ok );
    double dUTC = convert<double>( s, ok );
    if( !ok )
        yourTHROW( "Error reading CPF record 70.");

    YourTimeDateClass epoch = YourTimeDateClass::modJulianDate( mjd );
    epoch += YourTimeIntervalClass::seconds(secs);
    eop.epoch = epoch;
    eop.xPolar = RAD_PER_ARCSEC * xp;
    eop.yPolar = RAD_PER_ARCSEC * xp;
    eop.ut1_utc = YourTimeIntervalClass::seconds( dUTC);
    m_aEop.push_back( eop );
    return;
}
