#if ! defined( _OFFSET_H_ )
#define _OFFSET_H_

#include "YourTimeDateClass.h"
#include <string>
#include <vector>

//-----------------------------------------------------------------------------
//  Class Definition
//-----------------------------------------------------------------------------
//  This class provides a set of target offset data, primarily for
// use with CPF prediction file data sets, and especially for lunar targets.
// This class provides a way for targets to be defined as offsets from a
// precisely defined target (such as lunar centre).
//
// Remarks:
// Linear interpolation between nearest times is performed to calculate 
// parameters at a given epoch. No extrapolation is performed. Hence any
// request to obtain parameters at a time when no data is available covering
// that epoch will fail (i.e. the access method will return false).
//-----------------------------------------------------------------------------
class TargetOffsets
{
public:

    //  Constructor
    TargetOffsets();

    //  Destructor
    virtual ~TargetOffsets();

    // Structure to support Traget Offset data
    struct Record
    {
        Record()
        {
            epoch.setMJD(0,0);
            epochRx = epoch;
            xOut = yOut = zOut = 0.0;
            xIn  = yIn  = zIn  = 0.0;
        }
        YourTimeDateClass epoch;
        YourTimeDateClass epochRx;
        double xOut;
        double yOut;
        double zOut;
        double xIn;
        double yIn;
        double zIn;
    };

    // Method to determine outbound Target Offsets at a given epoch.
    // Return Value: True if valid values available. False otherwise, e.g. if no data available.
    bool getOut(YourTimeDateClass epoch, double& xOut, double& yOut, double& zOut) const;
    bool getIn (YourTimeDateClass epoch, double& xIn,  double& yIn,  double& zIn ) const;

    // Method to access name of target
    std::string name() const { return m_name; };

    // Method to determine if offset data is truly bidirectional.
    bool isBidirectional() const { return m_biDirectional;};

private:

    friend class PredictorCPF;

    //  Method to add a Targets Offset data record
    void addCpfRecord( const std::string& line );

    // Template function to convert a std::stringstream to a basic type.
    // Success indicated by the ok flag.
    template <class T>
    inline T convert( std::istringstream& iss, bool& ok)
    {
        T result; 
        iss.clear();
        if( !(iss >> result ) )
            ok = false;
        return result;
    }

    bool m_biDirectional;
    std::string m_name;
    long m_next;

    std::vector<Record> m_aOffsets;

};		// class TargetOffsets

#endif // _OFFSET_H_
