#if ! defined( _PREDICTIONGENERATOR_H_ )
#define _PREDICTIONGENERATOR_H_

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include "element.h"
#include "YourMatrix.h"
#include "YourTimeDateClass.h"
#include "Cpf.h"

#include "EOP.h"
#include "RotAngles.h"
#include "Offsets.h"

//-----------------------------------------------------------------------------
// Declarations
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//  Class Definition
//-----------------------------------------------------------------------------
//  A base class for object used to integrate, propagate, allocate
// or otherwise determine target position/ephemeris data at given epochs. 
// Ephemeris data will include at least position and velocity of a target at
// given points or nodes on the orbital path but may also include other data
// such as earth orientation parameters, transponder parameters, target offsets
// etc. All data is referred to the given epoch, whether it is defined by 
// an integration step size, arbitary fixed interval or available data. 
// All data will be output in the true geocentric earth body fixed or ITRF
// reference frame aligned with Greenwich prime meridian.
//
// Example: 
// See the derived classes for specific exapmles.
//-----------------------------------------------------------------------------
class PredictionGenerator
{

protected:

    // Constructor.
    // Arguments: stepSize -  Interval between nodes.
    //            element  -  An "element" object 
    PredictionGenerator( YourTimeIntervalClass stepSize, Element* element, EOP* eop = NULL )
    { 
        yourASSERTION(element); 
        m_element  = element;
        m_stepSize = stepSize;
        m_eop = eop;
        m_gmst = m_xPolar = m_yPolar = 0.0;
        m_ut1_utc = YourTimeIntervalClass::seconds(0);
        m_bidirectional = false;
    }

public:

    // Destructor
    virtual ~PredictionGenerator(){};

    //  Method to calculate the position and velocity at the next orbital node. 
    // Arguments: epoch - epoch of the next orbital node, ie. one interval after the last node.  
    virtual void calculateNextPosition( YourTimeDateClass &epoch)=0;

    // Method to calculate the position and velocity at the next orbital node. 
    // Arguments: epoch - nominal epoch for calculating the target position and velocity. 
    //                    Refer to derived classes.
    // Note: Use access methods to obtain the ephemeris data at the given epoch.
    virtual void calculatePosition( YourTimeDateClass epoch)=0;

    // A method to access the position component X in current reference frame.
    // Return Value: Position component along the X-axis in m or Km or Mm.
    double xm()  const {return 1.0e6 * m_posVelCor[0][0];};
    double xKm() const {return 1.0e3 * m_posVelCor[0][0];};
    double xMm() const {return         m_posVelCor[0][0];};

    // A method to access the position component Y in current reference frame.
    // Return Value: Position component along the Y-axis in m or Km or Mm.
    double ym()  const {return 1.0e6 * m_posVelCor[0][1];};
    double yKm() const {return 1.0e3 * m_posVelCor[0][1];};
    double yMm() const {return         m_posVelCor[0][1];};

    // A method to access the position component Z in current reference frame.
    // Return Value: Position component along the Z-axis in m or Km or Mm.
    double zm()  const {return 1.0e6 * m_posVelCor[0][2];};
    double zKm() const {return 1.0e3 * m_posVelCor[0][2];};
    double zMm() const {return         m_posVelCor[0][2];};

    // A method to access the velocity component Xdot.
    // Return Value: Velocity component along the X-axis in Mm/day or Km/sec.
    double umps()  const {return 1.0e6 * m_posVelCor[2][0]/SECS_PER_DAY ;};
    double uKmps() const {return 1.0e3 * m_posVelCor[2][0]/SECS_PER_DAY ;};
    double uMmpd() const {return         m_posVelCor[2][0];};

    // A method to access the velocity component Ydot.
    // Return Value: Velocity component along the Y-axis in Mm/day or Km/sec.
    double vmps()  const {return 1.0e6 * m_posVelCor[2][1]/SECS_PER_DAY ;};
    double vKmps() const {return 1.0e3 * m_posVelCor[2][1]/SECS_PER_DAY ;};
    double vMmpd() const {return         m_posVelCor[2][1];};

    // A method to access the velocity component Zdot.
    // Return Value: Velocity component along the Z-axis in Mm/day or Km/sec.
    double wmps()  const {return 1.0e6 * m_posVelCor[2][2]/SECS_PER_DAY;};
    double wKmps() const {return 1.0e3 * m_posVelCor[2][2]/SECS_PER_DAY;};
    double wMmpd() const {return         m_posVelCor[2][2];};

    // A method to access the earth orientation parameters at epoch.
    // Return Value: x and y polar motion in arcsecs and UT1-UTC. 
    double xPolar() const { return m_xPolar; };
    double yPolar() const { return m_yPolar; };
    YourTimeIntervalClass ut1_utc() const { return m_ut1_utc; };

    //===============
    // Access methods
    //===============

    // A method to access the matrix of position (Mm), velocity (Mm/d),
    // stellar abberation correction (m) and relativistic correction (ns)
    // ITRF node values at current epoch.
    const YourMatrix<double>& posVelCor() const { return m_posVelCor;};

    // A method to access the matrix of target offset (m) and rotation 
    // angle (degree) node values at current epoch.
    const YourMatrix<double>& targetOffsets() const { return m_offsets;};

    // A method to access the vector of accuracy (m) node values at current epoch.
    const std::vector<double>& accuracy() const{ return m_accuracy;};

    // A method to access the vector of transponder node values at current epoch.
    const std::vector<double>& transponder() const { return m_transponder;};

    //Access method for bidriectional flag.
    bool biDirectional() const {return m_bidirectional;};

    // A method to access the value for Greenwich Apparent Sidereal Time at epoch.
    YourTimeIntervalClass gast() const {return m_gast;};


protected:

    Element     *m_element;
    EOP         *m_eop;          // Earth orientation parameters
    YourTimeIntervalClass m_stepSize;     // stepSize
    YourTimeDateClass     m_epoch;        // current time
    YourTimeDateClass     m_initEpoch;    // initial (reference) epoch associated with given element.

    double m_xPolar; 
    double m_yPolar;
    YourTimeIntervalClass m_ut1_utc;
    YourTimeIntervalClass m_gast;
    double m_gmst;

    bool m_bidirectional;
    YourMatrix<double> m_posVelCor;     // A 7x3 matrix of positions, velocities and corrections
    YourMatrix<double> m_offsets;       // A 4x3 matrix of lunar target offsets and rotation angles
    std::vector<double> m_transponder;   // A 5 element vector of transponder data
    std::vector<double> m_accuracy;      // A 9 element vector of accuracy/runOff data

    YourMatrix<double> m_posm;
    YourMatrix<double> m_velm;
    YourMatrix<double> m_pm;

}; // class PredictionGenerator


//-----------------------------------------------------------------------------
//  Class Definition
//-----------------------------------------------------------------------------
// This class provides the association between Consolidated Prediction Format
// (CPF) table entries and nodes of a predicted track. Generally these will
// form a 1 to 1 mapping, but this class may be modified to support other
// arrangements.
//
// Remarks: None
//
// Example: The following represents a code fragment to outline the process 
//  
// do 
// {
//    Cpf cpf;
//    CpfAllocator orbitGen( cpf ... );
//    orbitGen.calculatePosition( startEpoch )
//    do for required period;
//    {
//       orbitGen.calculateNextPosition( epoch )
//       access and use ephemeris data referred to epoch
//    }
// }
//-----------------------------------------------------------------------------
class CpfAllocator : public PredictionGenerator
{

public:

    //  Constructor
    // Arguments: stepSize - Node interval. For this version, this interval should
    //                       exactly match the data interval used in the provided CPF file.
    //             element - Prediction data i.e. an entry from CPF table 
    CpfAllocator( YourTimeIntervalClass stepSize, Cpf* cpf, EOP* eop = NULL,
                  RotationAngles* rotAngles = NULL, TargetOffsets* targetOffsets = NULL );

    // : Destructor
    virtual ~CpfAllocator();

    // Method to update the "element" data.
    void setElement (Cpf* cpf);

    //  Method to calculate the position and velocity at the next orbital node. 
    // Arguments: epoch - calculated epoch of the next node, 1 time interval after the last node.  
    virtual void calculateNextPosition( YourTimeDateClass &epoch );

    //  A method to generate position and velocity predictions at a given time.
    // Arguments: epoch  -  nominal epoch for calculating the orbital position and velocity.
    // Note: Since the node is mapped to the available data, the actual epoch used after
    // this call will be at the closest node time prior to epoch, or the first available.
    // A call to calculateNextPosition provides the actual time.
    virtual void calculatePosition( YourTimeDateClass epoch );

private:

    RotationAngles*   m_rotAngles;
    YourTimeDateClass m_cpfEpoch;
    TargetOffsets*    m_targetOffsets;
    bool m_first;

};  // class CpfAllocator

//-----------------------------------------------------------------------------
//  Class Definition
//-----------------------------------------------------------------------------
 
// Predcition generator for other types of elements. For example, a
// Gauss-Jackson integrator for use with IRVs.
//-----------------------------------------------------------------------------
class OtherPredictionGenerator : public PredictionGenerator
{

};  

#endif // _PREDICTIONGENERATOR_H_
