#include "Predictor.h"
#include <math.h>
#include "yourException.h"


/******************************************************************************
  Constructor 1
******************************************************************************/
Predictor::Predictor(Track* track,
                     const std::string& objectName,
                     const std::string& directory )
{
    init( track );

    if( !directory.empty() && !objectName.empty() )
    {
        m_objectName = objectName;
        m_directory = directory;
        m_initOk = true;
    }
}

/******************************************************************************
  Constructor 2
******************************************************************************/
Predictor::Predictor(Track* track, long targetId, const std::string& directory)
{
    init( track );

    if( !directory.empty() && targetId > 0 )
    {
        m_directory = directory;
        m_targetId = targetId;
        m_initOk = true;
    }
}

/******************************************************************************
  Element Constructor 3
******************************************************************************/
Predictor::Predictor(Track* track, const Element& element)
{
    init( track );

    Element* e =  const_cast<Element&>(element).clone();
    e->copy( &const_cast<Element&>(element) );
    m_elements.push_back( e );
    m_initOk = true;
}

/******************************************************************************
******************************************************************************/
void Predictor::init( Track* track )
{
    m_initOk   = false;
    yourASSERTION( track );
    m_track = track;
    m_filename      = "";
    m_directory     = "";
    m_eopFilename   = "";
    m_objectName    = "";
    m_targetId   = 0; 
    return;
}

/******************************************************************************
  Destructor
******************************************************************************/
Predictor::~Predictor()
{
    try
    {
        deleteElements();
    }
    catch(...)
    {
        // Ensure that an exception is never thrown beyond the destructor
    }
}

/******************************************************************************
******************************************************************************/
void Predictor::deleteElements()
{
    if( !m_elements.empty() )
    {
        PElements::iterator elem;
        for( elem = m_elements.begin(); elem != m_elements.end(); elem++)
        {
            if( *elem != NULL )
            {
                delete (*elem);
                *elem = NULL;
            }
        }
    }
    return;
}

/******************************************************************************
 Method to load element data into the tle object.
******************************************************************************/
unsigned long Predictor::loadElements()
{
    // If a filename has been specified, just load that file.
    // The target Id is used to extract the relevent elements.
    if( !m_filename.empty() )
        readFile( m_filename, m_track->periodStart(), m_track->periodStop() );

    // Otherwise scan all files in the directory for the given target and extract elements
    else if( !m_directory.empty() )
    {
        // Use target information to define the associated filename. 
        // If necessary, the system may be changed to allow multiple files for
        // a given target, so this scheme will need to be changed.
        readFile( createFilename(m_directory, m_objectName),
                  m_track->periodStart(),
                  m_track->periodStop() );
    }
    else
        yourTHROW("Incomplete specification of required elements ");

    if(  m_elements.empty() )
        m_initOk = false;
    return unsigned long( m_elements.size() );
}


/******************************************************************************
  Get a record of element data given an index to the array of loaded elements.
  If the index exceeds available elements, it will return false.
******************************************************************************/
Element* Predictor::getElement( unsigned long index )
{
    Element *element;

    // If there are no elements already loaded, return NULL.
    if( m_elements.empty() )
        return NULL;
    if( index >= m_elements.size() || index < 0 )
        return NULL;

    // Extract data form the element set.
    element = m_elements[index];

    return element;               
}

/******************************************************************************
  Get an element record for the most recent date. This method searches a
  loaded element file for an element reference epoch equal to or immediately before
  the given epoch. If it cannot find a record, it will return a null record.
******************************************************************************/
Element* Predictor::getElement( YourTimeDateClass epoch, long *index )
{
    Element *element;
    if( index )
        *index = -1;

    // If there are no elements already loaded, return NULL.
    if( m_elements.empty() )
        return NULL;

    if(index) *index = m_elements.size()-1;

    // Search backwards throught the array to find an element whose time
    // is equal to or just before the requested time.
    std::vector<Element*>::reverse_iterator iter;
    for( iter = m_elements.rbegin(); iter < m_elements.rend(); iter++)
    {
        element = *iter;
        if( element->epoch() <= epoch )
        {
            if( m_targetId == 0 || element->targetId() == m_targetId )
            {
                return element;  
            }
        }
        if(index) (*index)--;
    }
    // If there is no element immediately preceding the given epoch,
    // return the first available element.
    element = m_elements[0];
    if(index) *index = 0;
    return element;               
}

