#if ! defined( _PREDICTOR_H_ )
#define _PREDICTOR_H_

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include "YourTimeDateClass.h"
#include <string>
#include <vector>
#include "Element.h"
#include "Track.h"


//-----------------------------------------------------------------------------
//  Class Definition
//-----------------------------------------------------------------------------
//  The Predictor class is the base class used by client applications to access
//  available orbital and other prediction "elements" necessary for the 
//  generation of various kinds of predicted target positions. 
//
// Remarks:
//  A client cannot directly construct an Predictor object. Only a descended 
//  class can be instantiated.
//
// Example: 
//
// Construct a "Track" object which will be the container for the resulting
// prediction data. Topocentric and other data can be extracted from the track
// object at whatever epoch is required. The track object also specifies the
// prediction period and node spacing.
//     Track track( startTime, stopTime, stepsize );
//     
// Construct a predictor of the required type. Some examples;
//     PredictorCPF predictor( track, 1234, "C:\\Predictions\\CPF" );
//
// Create predictions.
//     predictorCPF.generatePredictions();
//
// Finally use track object as required
//-----------------------------------------------------------------------------
class Predictor
{

protected :

    //  Constructor.
    // Arguments: track  - Track object to receive prediction data for a given period with a given node spacing.
    //        objectName - name of target or object to be tracked, as used in file convention for elements.
    //        targetId   - unique target identifier.
    //        directory  - directory containing required element data file(s) for the particular target.
    //        element    - the actual element to be used in generating prediction data.
    Predictor(Track* track, const std::string& objectName, const std::string& directory);
    Predictor(Track* track, long targetId, const std::string& directory);
    Predictor(Track* track, const Element& element);

public :

    //  Destructor.
    virtual ~Predictor();

    typedef std::vector<Element*> PElements;

    //  Methods to extract element data.
    // Arguments: index   -   Index (zero based) to list of loaded elements. The
    //                        element having this index will be returned.
    //            epoch   -   Time of required element. The element having a time equal to
    //                        or the most recent (prior) time will be returned.
    // Return Value: Element record containg required data. If no matching 
    // record can be found NULL is returned (index = -1 in this case).
    Element* getElement( unsigned long index );
    Element* getElement( YourTimeDateClass epoch, long *index = NULL );

    //  Method to explictly define the filename of the EOP file.
    void useEopFile(const std::string& filename ){ m_eopFilename = filename;};

    //  Method to generate predictions. The given track object will be 
    // populated using the predictions generated by the current elements and any 
    // subsidiary data required e.g. gravity model, earth orientation
    // parameters and timebias parameters as required. The appropriate orbit generator
    // will be used (depending on the given element type).
    // Return Value: True if predictions successfully generated. false otherwise.
    // Note: An inturrupt flag can be set to interrupt the prediction process if
    // and when required. 
    virtual bool generatePredictions(bool *interrupt = NULL) = 0;

protected :

    //  Method to load all of the required element data, in preparation for access 
    //  to individual element records. The method of loading this data depends on which
    // constructor is used. Use of an explicit element does not require this method,
    // but it can be called in any case.
    // Return Value: Number of elements loaded.
    unsigned long loadElements();

    // Method to delete elements created on the heap.
    void deleteElements();

    // Method to create the necessary filenames depending on the type of elements used.
    virtual std::string createFilename( const std::string& directory, const std::string& objectName ) = 0;

    // Method to read the file containing element data.
    virtual bool readFile( const std::string& filename, YourTimeDateClass start, YourTimeDateClass stop ) = 0; 

    // Initialisation method.
    void init( Track* track );

    // Class members
    bool m_useElement;
    std::string m_filename;
    std::string m_directory;
    std::string m_objectName;
    std::string m_eopFilename;

    long m_targetId;
    bool m_initOk;

    PElements m_elements;
    Track*  m_track;


};  // class Predictor


//-----------------------------------------------------------------------------
//  Class Definition
//-----------------------------------------------------------------------------
// The PredictorCPF class is the class used by client applications to access
// Consolidated Prediction Format elements which can then be used to create
// i.e. be allocated to prediction nodes containing target positions, 
// velocities, stallar aberrations, and other data. This format can be used
// for a wide variety of targets such as orbital satellites, lunar targets,
// synchronous and asynchronous transponders etc.
//-----------------------------------------------------------------------------
class PredictorCPF : public Predictor
{
public :

    //  Constructor (see base class).
    PredictorCPF( Track* track, const std::string& objectName, const std::string& directory );
    PredictorCPF( Track* track, long targetId, const std::string& directory);

    //  Destructor
    virtual ~PredictorCPF();

    //  Method to generate predictions. See base class.
    virtual bool generatePredictions(bool *interrupt = NULL);

protected :

    // Method to read records from a CPF file
    void loadFilesCPF();

    // Method to create the necessary filenames depending on the type of elements used.
    virtual std::string createFilename( const std::string& directory, const std::string& objectName );

    // Read contents of given CPF file for a given period. Data outside the given
    // period is ignored. 
    virtual bool readFile( const std::string& filename, YourTimeDateClass start, YourTimeDateClass stop ); 

    // Method to check and modity the node interval, ie. step size.
    bool checkNodeInterval( double headerInterval );

    // Method to create extended predictions using the last available CPF element.
    bool generateExtendedPredictions(bool *interrupt, Element* element = NULL );

    std::vector<PElements> m_elemSet;

};  // class CPF

//-----------------------------------------------------------------------------
//  Class Definition
//-----------------------------------------------------------------------------
//  Other predictor sub-classes. For example using IRVs.
//-----------------------------------------------------------------------------
class PredictorIRV : public Predictor
{

};  


#endif

 

