//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include "PredictionGenerator.h"
#include "Predictor.h"
#include "cpf.h"

EOP* l_eop = NULL;
TargetOffsets* l_offsets = NULL;
RotationAngles* l_rotAngles = NULL;

/******************************************************************************
  CPF Constructor 1
******************************************************************************/
PredictorCPF::PredictorCPF( Track* track, const std::string &objectName,
                            const std::string &directory) : Predictor(track, objectName, directory)
{
    loadElements();
};

/******************************************************************************
  Constructor 2
******************************************************************************/
PredictorCPF::PredictorCPF( Track* track, long targetId,
                            const std::string& directory ) : Predictor(track, targetId, directory)
{
    loadFilesCPF();
};

/******************************************************************************
 Destructor
******************************************************************************/
PredictorCPF::~PredictorCPF()
{
    try
    {
        if( !m_elemSet.empty() )
        {
            for( long i = 0; i < long( m_elemSet.size() ); i++)
            {
                m_elements = m_elemSet[i];
                deleteElements();
            }
        }
        if( l_eop )       delete l_eop;       l_eop = NULL;
        if( l_offsets )   delete l_offsets;   l_offsets = NULL;
        if( l_rotAngles ) delete l_rotAngles; l_rotAngles = NULL;
    }
    catch(...)
    {
        // Ensure that an exception is never thrown beyond the destructor
    }
}

/******************************************************************************
sort call back function
******************************************************************************/
bool greater(const Predictor::PElements& set1, const Predictor::PElements& set2)
{
    return (set2[0]->epoch() > set1[0]->epoch());
}

/******************************************************************************
******************************************************************************/
void PredictorCPF::loadFilesCPF( )
{
    std::string search = YourFile::makePath("", m_directory, "*", "*" );  
    std::string filename;
    std::string fname;
    m_initOk = false;

    WIN32_FIND_DATA fd;
    HANDLE hFile = FindFirstFile( search.c_str(), &fd );

	if (INVALID_HANDLE_VALUE != hFile)
	{
        bool ok = true;
		while( ok )
		{
            if( (FILE_ATTRIBUTE_DIRECTORY & fd.dwFileAttributes) != FILE_ATTRIBUTE_DIRECTORY) 
            {
                fname = fd.cFileName;
                filename = YourFile::makePath("", m_directory, fname, "");

                // Process this file.
                if( readFile( filename, m_track->periodStart(), m_track->periodStop() ) )
                {
                    if( !m_elements.empty() )
                    {
                        // Contruct an element Set on the heap
                        m_elemSet.push_back( m_elements );
                        m_initOk = true;
                    }
                }
            }
			ok = ( TRUE == FindNextFile( hFile, &fd ) );
		}
	    FindClose( hFile );

        // Reorder elements sets
        std::sort( m_elemSet.begin(), m_elemSet.end(), greater );
	}

    return;
}

/******************************************************************************
 Method to create an appropriate filename for the predictions file.
******************************************************************************/
std::string PredictorCPF::createFilename( const std::string& directory,
                                               const std::string& objectName )
{
    std::string filename;
    filename = YourFile::makePath("", directory, objectName, "cpf" );
    return filename;
}

/******************************************************************************
  Read contents of an CPF file and stores data for the given time period.
******************************************************************************/
bool PredictorCPF::readFile( const std::string& filename,
                             YourTimeDateClass start,
                             YourTimeDateClass stop ) 
{
    if( filename.empty() )
        yourTHROW("Invalid CPF filename provided.");
    bool corOk = false;
    bool eopOk = false;
    bool oscOk = false;
    bool first = false;
    m_elements.clear();

    try
    {
        // Try to open the file. Return false if it doesn't exists. 
        YourFile f;
        if( !f.fopen( filename.c_str(), std::ios::in) )
            return false;

        // Read one line at a time up to a maximum of 100 characters
        f.setLines(1, 100);
        std::string line;

        // Header and optional synchronous data and asynchronous data
        Cpf record;
        if( !m_objectName.empty() )
            record.objectNameIs( m_objectName );

        Cpf::Header header;
        Cpf* cpf = NULL;
        do
        {
            // Stream in next line in the file.
            f >> line;

            // Stream in first field.
            bool ok = true;
            std::istringstream s(line);
            std::string hh = convert<std::string>(s, ok);
            if( ok )
            {
                switch( Cpf::recordType( hh ) )
                {
                    case Cpf::RecordType::headerBasic1 :
                        Cpf::parseBasicHeader1( line, &header );
                        break;
                    case Cpf::RecordType::headerBasic2:
                    {
                        Cpf::parseBasicHeader2( line, &header );

                        // Check that we have data for the required target
                        if( m_targetId != 0 && header.identifier != m_targetId )
                            return false;
                        break;
                    }
                    case Cpf::RecordType::lastHeader:
                        Cpf::parseLastHeader( line, &header );
                        break;
                    case Cpf::RecordType::recordVelocity :
                        record.parseVelocityRecord( line, header );
                        break;
                    case Cpf::RecordType::recordCorrection :
                        record.parseCorrectionRecord( line );
                        corOk = true;
                        break;
                    case Cpf::RecordType::lastRecord :
                        record.parseLastRecord( line );
                        break;

                    case Cpf::RecordType::headerAccuracy:
                    {
                        Cpf::parseAccuracyHeader( line, &header );
                        break;
                    }
                    case Cpf::RecordType::headerTransponder:
                    {
                        Cpf::parseTransponderHeader( line, &header );
                        break;
                    }
                    case Cpf::RecordType::headerCoM:
                    {
                        Cpf::parseCentreOfMassHeader( line, &header );
                        break;
                    }
                    case Cpf::RecordType::recordTransponder :
                    {
                        record.parseTransponderRecord( line, header ); 
                        oscOk = true;
                        break;
                    }

                    case Cpf::RecordType::recordPosition :
                    {
                        // If this is a new outbound record then save the current
                        // valid record. Then start with a clean Cpf record.
                        long dir = convert<long>( s, ok); if( !ok ) break;
                        if( record.valid() && ( dir == 0 || dir == 1) )
                        {
                            // If record falls within a given extended period, add to the array
                            if( record.epoch() >= start - YourTimeIntervalClass::days( 1, 0) && 
                                record.epoch() <= stop  + YourTimeIntervalClass::days( 1, 0) )
                            {
                                record.addHeaderData( header ); 
                                cpf = new Cpf;
                                *cpf = record;
                                m_elements.push_back( cpf );
                            }
                            else
                            {
                                // Have we read a record past the required date period ?
                                if( record.epoch() > stop )
                                    goto exit;
                            }
                            // If data is outbound record, then restart with a clean record
                            Cpf initRecord;
                            record = initRecord;
                            record.objectNameIs( m_objectName );
                        }
                        record.parsePositionRecord( line, header );
                        break;
                    }

                    case Cpf::RecordType::recordOffset :
                    {
                        // Add a new target offset record to the Offsets container
                        if( !l_offsets )
                            l_offsets = new TargetOffsets;
                        l_offsets->addCpfRecord( line );
                        break;
                    }

                    case Cpf::RecordType::recordEOP :
                    {
                        // Add a new Eop record to the EOP container
                        if( !l_eop )
                            l_eop = new EOP;
                        l_eop->addCpfRecord( line );
                        eopOk = true;
                    }

                    case Cpf::RecordType::recordAngle :
                    {
                        // Add a new rotation angle record to the Angles container
                        if( !l_rotAngles )
                            l_rotAngles = new RotationAngles;
                        l_rotAngles->addCpfRecord( line );
                        l_rotAngles->rotationAngleTypeIs( header.angleType );
                        break;
                    }

                    // If this record is not recognizable, try next line.
                    default: break;
                }
            }
        } while( !f.eof() );

        // Perform checks if any data found
exit:   if( !m_elements.empty() )
        {
            // Check that record intervals are consistent with header
            if( !checkNodeInterval( header.interval ) )
                yourTHROW(" Record intervals inconsistent with header.");

            // Check we have corrections records (30) for all target types except passive satellite
            if( header.targetType != Cpf::TargetType::passiveSatellite && !corOk )
                yourTHROW(" No corrections records for distant targets.");

            // Check we have eop data for or inertialTOD, inertialJ2000 reference frames
            if( header.referenceFrame != Cpf::ReferenceFrame::itrf && !eopOk )
                yourTHROW(" No eop records for inertial reference frame transformations.");

            // Check asynch transponder has osc relativistic corr (40)
            if( header.targetType == Cpf::TargetType::asynchTransponder && !oscOk )
                yourTHROW(" No oscillator corrections for asynchronous transponder targets.");
            return true;
        }
    }
 
    catch (Exception& e)
    {
        yourTHROW( e.description() );
    }
    return false;
}


/******************************************************************************
 Check for consistency between the specified interval and record intervals
******************************************************************************/
bool PredictorCPF::checkNodeInterval( double headerInterval )
{
    YourTimeIntervalClass interval = YourTimeIntervalClass::seconds( headerInterval, 0);

    // Check if the CPF table is specified to have entries at constant time intervals
    if( headerInterval != 0 )
    {
        // Check time intervals between records
        std::vector<Element*>::iterator i;
        long n = 0;
        for( i = m_elements.begin() + 1; i < m_elements.end(); i++ )
        {
            double dt = YourTimeIntervalClass::seconds( (*i)->epoch() - (*(i-1))->epoch() );
            if( fabs( dt - headerInterval ) > 0.1 ) n++; 
        }
        // Report any records deviating from specified interval by more than 0.1 seconds.
        std::ostringstream os;
        if( n > 0 )
        {
            os << "Warning - " << n << " inconsistent record intervals.";
            generateAlert( os.str() );
        }

        // If we have too many deviations, then return false.
        if( n > 20 )
            return false;

        // Everything OK so reset step size to that given by the CPF file.
        m_track->setStepSize( interval );
        return true;
    }

    // The header interval is zero, so we may have variable spaced data.
    // Handling such data is TBD.
    return true;
}

/******************************************************************************
 This routine calculates predictions over a given period, which may be many
 days, or just a few mins. It may use one or many elements (i.e. CPF records )
******************************************************************************/
bool PredictorCPF::generatePredictions(bool *interrupt)
{
    if( ! m_initOk || m_elemSet.empty() )
        return false;

    // Adjust the start and end times of the prediction period by a number 
    // of integration stepsizes to allow initial and final processing to settle.
    // As a minimum, allow for the width of the interpolation filter, plus a couple
    // or at least 10 * stepsizes.
    long numberOfnodes = m_track->interpolatorWidth() + 2;
    if( numberOfnodes < 10 )
        numberOfnodes = 10;
    double stepSize =  YourTimeIntervalClass::seconds( m_track->nodeInterval() );
    YourTimeIntervalClass offset = YourTimeIntervalClass::seconds(stepSize * numberOfnodes);
    YourTimeDateClass startPeriod = m_track->periodStart() - offset;
    YourTimeDateClass stopPeriod  = m_track->periodStop()  + offset;

    // Cycle through each the element Sets
    long lastSet = long( m_elemSet.size() - 1);
    for( long set = 0; set <= lastSet ; set++ )
    {
        m_elements = m_elemSet[set];

        // Obtain the initial element.
        long index;
        Cpf* d1 = NULL;
        Cpf* d2 = NULL;
        d1 = static_cast<Cpf*>( getElement( startPeriod, &index ) );

        if( index < 0 )
            yourTHROW;

        // Initialise the predictor generator (i.e. the CPF data allocator) with the element
        // prior to startPeriod or the first available element.
        // Cycle through the available elements, mapping Cpf records to predictions.
        CpfAllocator predictor(m_track->nodeInterval(), d1, l_eop, l_rotAngles, l_offsets );
        predictor.calculatePosition( startPeriod );
        bool newElement = true;

        // Extract elements (entries in CPF file) during the period, and provide results at time
        // intervals defined by the element epoch, which should match the step size or node interval.
        // Stop when the end of the period is reached.
        YourTimeDateClass time;
        do
        {
            if( interrupt && *interrupt )
                    return false;

            // Get next available element.
            d1 = d2;
            d2 = static_cast<Cpf*>( getElement( ++index ) );
            if( !d2 )
            {
                if( set == lastSet )
                    generateExtendedPredictions(interrupt, d1 );
                break;
            }
            predictor.setElement( d2 );
            predictor.calculateNextPosition( time );
            if( time >= startPeriod  )
            {
                if( m_track->trackType() == Track::Type::lunar )
                {
                    LunarTrack* trk = static_cast<LunarTrack*>(m_track);
                    trk->appendNode( time, newElement, predictor.biDirectional(), d2->lightTimeCorrected(),
                                    predictor.posVelCor(), predictor.xPolar(), predictor.yPolar(),
                                    predictor.ut1_utc(), predictor.gast(), predictor.targetOffsets() );
                }
                else if( m_track->trackType() == Track::Type::transponder )
                {
                    TransponderTrack* trk = static_cast<TransponderTrack*>(m_track);
                    trk->appendNode( time, newElement, predictor.biDirectional(), d2->lightTimeCorrected(),
                                     predictor.posVelCor(), predictor.xPolar(), predictor.yPolar(),
                                     predictor.ut1_utc(), predictor.transponder() );
                }
                else
                {
                    // Add a new point to an array stored in m_predicts.
                    m_track->appendNode( time, newElement, predictor.biDirectional(), d2->lightTimeCorrected(),
                                        predictor.posVelCor(), predictor.xPolar(), predictor.yPolar(),
                                        predictor.ut1_utc() );
                }
                newElement = false;
            }
        } while( d2 && time <= stopPeriod );
    }
    return true;
}

/******************************************************************************
******************************************************************************/
bool PredictorCPF::generateExtendedPredictions(bool *interrupt, Element* element )
{
    Cpf* cpf = static_cast<Cpf*>( element );

    if( !cpf->irvCompatible() )
        return false;

    // Construct an IRV element, irv, using fields from the given CPF element.
    // Then create an IRV predictor and continue generating predictions.
    // Details not given.
    PredictorIRV p( m_track, irv );

    p.generatePredictions( interrupt );

    return true;
}
