//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include "RotAngles.h"
#include "yourException.h"
#include <math.h>

/******************************************************************************
 Constructor 
******************************************************************************/
RotationAngles::RotationAngles()
{
}

/******************************************************************************
 Destructor
******************************************************************************/
RotationAngles::~RotationAngles()
{
}

/******************************************************************************
 This subroutine uses linear interpolation to obtain target offset values
 at a given epoch.
 The function returns FALSE if the caller supplies an epoch outside the 
 range of available data. The function does not extrapolate data.
******************************************************************************/
bool RotationAngles::get(YourTimeDateClass epoch, double& angle1,
                         double& angle2, double& angle3, double& gast )const
{
    gast = angle1 = angle2 = angle3 = 0.0;

    if( m_aAngles.empty() )
        return false;

    std::vector<RotationAngles::Record>::const_iterator i = m_aAngles.begin();
    if( m_aAngles.size() == 1 )
    {
        angle1 = i->angle1;
        angle2 = i->angle2;
        angle3 = i->angle3;
        gast   = i->gast;
        return true;
    }
    else
    {
        for( ; i < m_aAngles.end(); i++ )
        {
            double d1 = YourTimeIntervalClass::days(        epoch - i->epoch );
            double d2 = YourTimeIntervalClass::days( (i+1)->epoch -    epoch );
            double d3 = YourTimeIntervalClass::days( (i+1)->epoch - i->epoch );
            if( fabs(d1) <= fabs(d3) && fabs(d2) <= fabs(d3) )
            {
                double f = 0.0;
                if( fabs(d3) > 0.0 ) 
                    f = d1 / d3;
                angle1 =  ( (i+1)->angle1 - i->angle1 ) * f + i->angle1;
                angle2 =  ( (i+1)->angle2 - i->angle2 ) * f + i->angle2;
                angle3 =  ( (i+1)->angle3 - i->angle3 ) * f + i->angle3;
                gast   =  ( (i+1)->gast   - i->gast )   * f + i->gast;
                return true;
            }
        }
    }

    return false;
}

/******************************************************************************
Record type 60 Rotation angle of offset (Surface features) 
(See Rotation Angle Type in header record 2.)

A2     Record Type (= "60")
I5     Modified Julian Date (MJD)
F13.6  Seconds of Day (UTC)
F17.12 Rotation angle 1 in degrees (For moon, euler angle phi)
F17.12 Rotation angle 2 in degrees (For moon, euler angle theta)
F17.12 Rotation angle 3 in degrees (For moon, euler angle psi)
F17.12 Greenwich Apparent Sidereal Time in hours
******************************************************************************/
void RotationAngles::addCpfRecord( const std::string& line )
{
    std::istringstream s;
    s.str( line );

    bool ok = true;
    std::string recType = convert<std::string>( s, ok );
    if( recType != "60" )
        yourTHROW;

    double mjd  = convert<double>( s, ok );
    double secs = convert<double>( s, ok );
    double xRot = convert<double>( s, ok );
    double yRot = convert<double>( s, ok );
    double zRot = convert<double>( s, ok );
    double gast = convert<double>( s, ok );
    if( !ok )
        yourTHROW( "Error reading CPF record 60.");

    Record rotAngle;
    YourTimeDateClass epoch = YourTimeIntervalClass::modJulianDate( mjd, 0 );
    epoch += YourTimeIntervalClass::seconds(secs, 15);

    rotAngle.epoch  = epoch;
    rotAngle.angle1 = xRot;
    rotAngle.angle2 = yRot;
    rotAngle.angle3 = zRot;
    rotAngle.gast   = gast;

    m_aAngles.push_back( rotAngle );
    return;
}

/******************************************************************************
 This subroutine uses linear interpolation or extrapolation
 of rotation angles data at a given epoch. 
 A default zero is returned if no data is available. 
******************************************************************************/
double RotationAngles::gast( YourTimeDateClass epoch ) const
{
    double gast;
    double a1, a2, a3;
    if( get( epoch, a1, a2, a3, gast ) )
        return gast;
    else
        return 0.0;
}
