#include "SiteHandle.h"
#include <math.h>

/******************************************************************************
 Constructor
******************************************************************************/
SiteHandle::SiteHandle( const std::string& name,
                        double geodeticLatitude,
                        double longitude,
                        double height,
                        double horizon,
                        double x, double y, double z )
{
    m_name = name;

    // Convert degrees to radians, m to Mm.
    m_longitude         = RAD_PER_DEG * longitude;
    m_geodeticLatitude  = RAD_PER_DEG * geodeticLatitude;
    m_horizon           = RAD_PER_DEG * horizon;
    m_height            = 1.0e-6      * height;
    double geocentricLatitude = m_geodeticLatitude;

    m_posLocal[0] = x;
    m_posLocal[1] = y;
    m_posLocal[2] = z;

    double sinLong = sin( m_longitude );
    double cosLong = cos( m_longitude );
    double sinLat  = sin( m_geodeticLatitude);
    double cosLat  = cos( m_geodeticLatitude);

    double e2 = ( 2.0 - 1.0/EarthFlattening )/ EarthFlattening;
    double N = EarthEquatorialRadius / sqrt(1.0 - e2 * sinLat * sinLat);

    if( cosLat != 0 )
        geocentricLatitude =  atan2( (1.0 - N * e2/( N + height ) ) * sinLat, cosLat );
    
    // Calculate true body fixed (ITRF) coordinates
    m_posITRF[0] = ( N + m_height )          * cosLat * cosLong;
    m_posITRF[1] = ( N + m_height )          * cosLat * sinLong;
    m_posITRF[2] = ( N + m_height - e2 * N ) * sinLat;

    double radius = sqrt( m_posITRF[0] * m_posITRF[0] + 
                          m_posITRF[1] * m_posITRF[1] +
                          m_posITRF[2] * m_posITRF[2] );

    m_posSpherical[0] = radius;
    m_posSpherical[1] = m_longitude;
    m_posSpherical[2] = geocentricLatitude;


    // Establish a history of elevation values for each registered site.
    m_previousAz = 0.0;
    m_previousIndex = -1;
    m_segment = -1;
    m_index = -1;

    for(long i=0; i < 6; i++)
    {
        m_targetPosRect[i] = m_targetPosSpher[i] = 0.0;
        m_targetVelRect[i] = m_targetVelSpher[i] = 0.0;
        m_targetOffset[i]  = m_aberCor[i]        = 0.0;
        m_targetMeanSpher[i] = 0.0;
    }



    m_rotAngle[0] = m_rotAngle[1]  = m_rotAngle[2] = m_transponderCor = 0.0;
    m_xPolar = m_yPolar  = m_targetRA  = m_targetDec = m_targetVelocityAzEl = 0.0;
    m_targetRange  = m_ut1_utc = m_gast = m_relCor[0] = m_relCor[1] = YourTimeDateClass::seconds(0); 
    m_targetIlluminated = m_targetNearSun = m_targetNearMoon = false;
    m_targetMagnitude = 100000000.0;
}


/******************************************************************************
 Destructor
******************************************************************************/
SiteHandle::~SiteHandle( )
{
}

