      SUBROUTINE SATIL(WJD, SEC, LONG, GDLAT, AZR, ELR, SEPD)
C     =======================================================
C
C  Compute the angular separation between an earth satellite and the sun.
C
C  Input:
C	WJD	- Julian date of satellite prediction
C	SEC	- Seconds into day of prediction
C	LONG	- Longitude of observing site in radians
C	GDLAT	- Geodetic latitude of observing site in radians
C	AZR	- Azimuth of satellite at site in radians
C	ELR	- Elevation of satellite at site in radians
C
C  Output:
C	SEPD	- Angular separation of satellite and sun in degrees.
C
C=====================================================================

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /MATH/    PIO2, PI, TWOPI, RDPDG, RDPHR, RDPAS, RDPTS
      COMMON /COMJC/  TJ,TJ2,TJ3
 
      DOUBLE PRECISION WJD, SEC, LONG, GDLAT, AZR, ELR, SEP, SEPD
 
      SINPHI = DSIN(GDLAT)
      COSPHI = DCOS(GDLAT)

      DJ= WJD+ SEC/86400.D0
      CALL JCENT(DJ)
      TC=1.0D0+TJ
      SID= DLSID(DJ,LONG)
C
C=====================================================================
C       AZIMUTH AND ZENITH DISTANCE OF MEAN SUN
C=====================================================================
C
C  Compute solar longitude and obliquity to the ecliptic
      CALL SUNTRUE(TC,SUNLONG,OBL)

C Convert these to right ascension and dclination
      ALFSUN = DATAN2(DSIN(SUNLONG)*DCOS(OBL),DCOS(SUNLONG))
      DECSUN = DASIN (DSIN(SUNLONG)*DSIN(OBL))

C Using Sidereal time, convert ra to Hour Angle and from there to az and
C   zenith distance
      HSUN   = SID-ALFSUN
      ZENSUN = DACOS(DSIN(DECSUN)*SINPHI+DCOS(DECSUN)*COSPHI*DCOS(HSUN))
      ZSUND  = ZENSUN/RDPDG
      AZSUN  = DATAN2(-DCOS(DECSUN)*DSIN(HSUN),(DSIN(DECSUN)*COSPHI -
     @                                 DCOS(DECSUN)*SINPHI*DCOS(HSUN)))
*
C Using solar and satellite az and z.d., determine the angular separation
      ELSATD = ELR/RDPDG
      ELSUND = 90.0D0-ZSUND
      ZSAT   = 0.5D0*PI-ELR
      ZSATD  = ZSAT/RDPDG
      AZSATD = AZR/RDPDG
      IF(AZSATD.LT.0.0D0) AZSATD=AZSATD+360.0
      AZSUND=AZSUN/RDPDG
      IF(AZSUND.LT.0.0D0) AZSUND=AZSUND+360.0
      SEP = DACOS(DCOS(ZSAT)*DCOS(ZENSUN)+DSIN(ZSAT)*DSIN(ZENSUN)*
     @                                            DCOS(AZR-AZSUN))
      SEPD=SEP/RDPDG

      RETURN
      END
 
**************************************************************************
**************************************************************************
C
C
      DOUBLE PRECISION FUNCTION DLSID(DJ,LONG)
C     ========================================
C
C*********************************************************************
C*    CALCULATES LOCAL APPARENT SIDEREAL TIME IN TERMS OF UT1 (i.e.
C*    LAST1 IN MUELLER'S NOTATION) AT A PLACE WHOSE LONGITUDE IS GIVEN
C*    AS THE SECOND ARGUMENT. See pp. 50-53 of Explanatory Supplement to
C*    the Astronomical Almanac (1992 ed.).
C*    Note: this appears to be a low precision conversion... rlr.
C*    INPUTS :
C*	DJ	- Julian date
C*	LONG	- Longitude of site in radians
C*    OUTPUT :
C*	DLSID	- Local Mean Sidereal Time in radians
C*********************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /MATH/ PIO2, PI, TWOPI, RDPDG, RDPHR, RDPAS, RDPTS
      COMMON/COMJC/ TJ,TJ2,TJ3
 
      DOUBLE PRECISION LONG
 
      GSTC = ((6.0D0*60.0D0+41.0D0)*60.0D0+50.54841)*RDPTS
      GST1 = 8640184.812866D0*RDPTS
      GST2 = 0.093104D0*RDPTS
      GST3 = -6.2D-06*RDPTS
C?      CALL JCENT(DJ)
      DDST  = DMOD(DJ+0.5d0,1.d0)*TWOPI +
     *	      LONG+GSTC+GST1*TJ+GST2*TJ2+GST3*TJ3
      DLSID = DMOD(DDST,TWOPI)
      IF(DLSID.LT.0.0D0) DLSID=DLSID+TWOPI
      RETURN
      END
C
**************************************************************************
**************************************************************************
C
      SUBROUTINE JCENT(DJ)
C     ====================
C
C	Calculate the number of Julian centuries between the input date 
C	and 1900.0.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
           COMMON/COMJC/TJ,TJ2,TJ3
      TJ  = (DJ-2451545.0D0)/36525.0D0
      TJ2 = TJ*TJ
      TJ3 = TJ2*TJ
      RETURN
      END
C
**************************************************************************
**************************************************************************
C
C
      SUBROUTINE SUNTRUE(TC,SUNLONG,OBL)
C     ==================================
C
C*********************************************************************
C       CALCULATES TRUE LONGITUDE OF SUN RELATIVE TO MEAN EQUINOX OF
C       DATE, AND OBLIQUITY OF ECLIPTIC (OBL).
C       INPUT  : TC - JULIAN CENTURIES SINCE 1900.0
C       OUTPUT : SUNLONG - TRUE LONG. OF SUN IN RADIANS
C                OBL     - OBLIQUITY IN RADIANS
C*********************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /MATH/ PIO2, PI, TWOPI, RDPDG, RDPHR, RDPAS, RDPTS
C
C---------------------------------------------------------------------
C       SUN'S MEAN ANOMALY(ANOM), LONGITUDE OF PERIGEE(PER) & OBLIQUITY
C       FROM FORMULAE IN EXPLANATORY SUPPLEMENT
C---------------------------------------------------------------------
C
      ANOM=(358.475833D0+35999.04975D0*TC- 0.00015D0*TC*TC)*RDPDG
      PER =(281.220844D0+   1.719175D0*TC-0.000453D0*TC*TC)*RDPDG
      OBL =( 23.452294D0-  0.0130125D0*TC-.0000016D0*TC*TC)*RDPDG
      ECCEN=0.01675104D0-0.00004180*TC
C
C---------------------------------------------------------------------
C       TRUE ANOMALY FROM MEAN ANOMALY USING EQUATION OF THE CENTRE
C       AND THEN ADD TRUE ANOMALY TO LONGITUDE OF PERIGEE
C---------------------------------------------------------------------
C
      TRUE=ANOM+(2.0*ECCEN-0.25*(ECCEN**3))*DSIN(ANOM)+1.25*(ECCEN**2)*
     @     DSIN(2.0D0*ANOM)+1.083333*(ECCEN**3)*DSIN(3.0D0*ANOM)
      SUNLONG=TRUE+PER
      RETURN
      END

